@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from __future__ import unicode_literals
 import six
 import random
 import string
@@ -8,35 +9,35 @@ import re
 import copy
 import datetime
 from collections import OrderedDict
-try:
-    import urlparse
-except ImportError:
-    import urllib.parse as urlparse
+from six.moves import urllib_parse as urlparse, range
 from IPy import IP, MAX_IPV4_ADDRESS, MAX_IPV6_ADDRESS
 from . import exceptions
-
+from .utils import force_text
+from .translation import gettext as _
 
 __all__ = [
-    'BaseField',
-    'StringField',
-    'NumberField',
-    'IntegerField',
-    'FloatField',
-    'BoolField',
-    'UUIDField',
-    'MD5Field',
-    'SHAField',
-    'EmailField',
-    'IPAddressField',
-    'URLField',
-    'EnumField',
-    'DictField',
-    'ListField',
-    'TimestampField',
-    'DatetimeField',
-    'DateField',
+    # Don't need to add field to here by hand,
+    # BaseFieldMetaClass will auto add field to here.
 ]
 
+FIELDS_NAME_MAP = {
+    # Don't need to add field to here by hand,
+    # BaseFieldMetaClass will auto add field to here.
+}
+
+
+def create_field(field_info):
+    """
+    Create a field by field info dict.
+    """
+    field_type = field_info.get('type')
+    if field_type not in FIELDS_NAME_MAP:
+        raise ValueError(_('not support this field: {}').format(field_type))
+    field_class = FIELDS_NAME_MAP.get(field_type)
+    params = dict(field_info)
+    params.pop('type')
+    return field_class.from_dict(params)
+
 
 class EmptyValue(object):
     """
@@ -56,6 +57,18 @@ class EmptyValue(object):
 EMPTY_VALUE = EmptyValue()
 
 
+class BaseFieldMetaClass(type):
+
+    def __new__(cls, name, bases, attrs):
+        __all__.append(name)
+        clazz = super(BaseFieldMetaClass, cls).__new__(cls, name, bases, attrs)
+        field_name = attrs.get('FIELD_TYPE_NAME')
+        if field_name is not None and field_name != 'object':
+            FIELDS_NAME_MAP[field_name] = clazz
+        return clazz
+
+
+@six.add_metaclass(BaseFieldMetaClass)
 class BaseField(object):
     """ 
     BaseField
@@ -99,7 +112,7 @@ class BaseField(object):
     @classmethod
     def _check_value_range(cls, min_value, max_value):
         if max_value is not None and max_value < min_value:
-            raise ValueError('the max value must greater than or equals the min value, got min value={min}, max value={max}'.format(
+            raise ValueError(_('the max value must greater than or equals the min value, got min value={min}, max value={max}').format(
                 min=min_value, max=max_value))
 
     def _convert_type(self, value):
@@ -121,23 +134,13 @@ class BaseField(object):
         """
         Collect all PARAMS from this class and its parent class.
         """
-        params = []
-        parent_cls = cls
-        while issubclass(parent_cls, BaseField):
-            params.extend(parent_cls.PARAMS)
-            bases = parent_cls.__bases__
-            parent_cls = bases[0]
+        params = list(cls.PARAMS)
+        bases = cls.__bases__
+        for base in bases:
+            if issubclass(base, BaseField):
+                params.extend(base._get_all_params())
         return params
 
-    @property
-    def _params_and_values(self):
-        params = self._get_all_params()
-        params_and_values = {}
-        for name in params:
-            if hasattr(self, name):
-                params_and_values[name] = getattr(self, name)
-        return params_and_values
-    
     def validate(self, value):
         """
         return validated value or raise FieldValidationError.
@@ -161,13 +164,13 @@ class BaseField(object):
         if not isinstance(value, self.INTERNAL_TYPE):
             if self.strict:
                 raise exceptions.FieldValidationError(
-                    'got a wrong type: {0}, expect {1}'.format(type(value).__name__, self.FIELD_TYPE_NAME))
+                    _('got a wrong type: {0}, expect {1}').format(type(value).__name__, self.FIELD_TYPE_NAME))
             else:
                 try:
                     value = self._convert_type(value)
                 except (ValueError, TypeError) as e:
                     raise exceptions.FieldValidationError(
-                        'type convertion is failed: {0} -> {1}'.format(type(value).__name__, self.FIELD_TYPE_NAME))
+                        _('type convertion({0} -> {1}) is failed: {2}').format(type(value).__name__, self.FIELD_TYPE_NAME, str(e)))
         return value
 
     def is_required(self):
@@ -201,9 +204,26 @@ class BaseField(object):
         d = {
             'type': self.FIELD_TYPE_NAME,
         }
-        d.update(self._params_and_values)
+        params = self._get_all_params()
+        for name in params:
+            if hasattr(self, name):
+                value = getattr(self, name)
+                # 处理特殊值
+                if value is EMPTY_VALUE:
+                    value = '__empty__'
+                d[name] = value
         return d
 
+    @classmethod
+    def from_dict(cls, params):
+        """
+        Create a field from params.
+        sub-class can override this method.
+        """
+        if params.get('default') == '__empty__':
+            params['default'] = EMPTY_VALUE
+        return cls(**params)
+
     def mock_data(self):
         """
         reutrn mocking data
@@ -243,14 +263,14 @@ class StringField(BaseField):
 
         if len(value) < self.min_length:
             raise exceptions.FieldValidationError(
-                'string is too short, min-lenght is {}'.format(self.min_length))
+                _('string is too short, min-lenght is {}').format(self.min_length))
         if self.max_length and len(value) > self.max_length:
             raise exceptions.FieldValidationError(
-                'string is too long, max-lenght is {}'.format(self.max_length))
+                _('string is too long, max-lenght is {}').format(self.max_length))
 
         if not self._match(value):
             raise exceptions.FieldValidationError(
-                '{0} not match {1}'.format(self.regex.pattern, value))
+                _('{0} not match {1}').format(self.regex.pattern, value))
 
         return value
 
@@ -270,7 +290,7 @@ class StringField(BaseField):
             max_ = min_ + 100
         size = random.randint(min_, max_)
         random_str = ''.join(
-            [random.choice(string.letters + string.digits) for _ in xrange(size)])
+            [random.choice(string.ascii_letters + string.digits) for _ in range(size)])
         random_str = self.to_internal(random_str)
         return random_str
 
@@ -295,11 +315,11 @@ class NumberField(BaseField):
 
         if self.min_value is not None and value < self.min_value:
             raise exceptions.FieldValidationError(
-                'value is too small, min-value is {}'.format(self.min_value))
+                _('value is too small, min-value is {}').format(self.min_value))
 
         if self.max_value is not None and value > self.max_value:
             raise exceptions.FieldValidationError(
-                'value is too big, max-value is {}'.format(self.max_value))
+                _('value is too big, max-value is {}').format(self.max_value))
 
         return value
 
@@ -355,7 +375,7 @@ class UUIDField(BaseField):
         format: what format used when to_presentation, supports 'hex', 'str', 'int', 'bytes', 'bytes_le'
         """
         if format not in self.SUPPORT_FORMATS:
-            raise ValueError('not supports format: {}'.format(format))
+            raise ValueError(_('not supports format: {}').format(format))
         self.format = format
 
         kwargs.setdefault('strict', False)
@@ -393,7 +413,7 @@ class MD5Field(StringField):
             return super(MD5Field, self)._validate(value)
         except exceptions.FieldValidationError as e:
             raise exceptions.FieldValidationError(
-                'Got wrong md5 value: {}'.format(value))
+                _('Got wrong md5 value: {}').format(value))
 
     def mock_data(self):
         return ''.join([random.choice(string.hexdigits) for i in range(32)])
@@ -406,12 +426,12 @@ class SHAField(StringField):
 
     def __init__(self, version=256, **kwargs):
         if version not in self.SUPPORT_VERSION:
-            raise ValueError('{0} not support, support versions are: {1}'.format(
+            raise ValueError(_('{0} not support, support versions are: {1}').format(
                 version, self.SUPPORT_VERSION))
         if version == 1:
             length = 40
         else:
-            length = version / 8 * 2
+            length = int(version / 8 * 2)
         self.version = version
         self.length = length
         kwargs['strict'] = True
@@ -426,7 +446,7 @@ class SHAField(StringField):
             return super(SHAField, self)._validate(value)
         except exceptions.FieldValidationError as e:
             raise exceptions.FieldValidationError(
-                'Got wrong sha{0} value: {1}'.format(self.version, value))
+                _('Got wrong sha{0} value: {1}').format(self.version, value))
 
     def mock_data(self):
         return ''.join([random.choice(string.hexdigits) for i in range(self.length)])
@@ -446,11 +466,12 @@ class EmailField(StringField):
             return super(EmailField, self)._validate(value)
         except exceptions.FieldValidationError as e:
             raise exceptions.FieldValidationError(
-                'Got wrong email value: {}'.format(value))
+                _('Got wrong email value: {}').format(value))
 
     def mock_data(self):
-        name = ''.join(random.sample(string.lowercase, 5))
-        domain = '{0}.com'.format(''.join(random.sample(string.lowercase, 3)))
+        name = ''.join(random.sample(string.ascii_lowercase, 5))
+        domain = '{0}.com'.format(
+            ''.join(random.sample(string.ascii_lowercase, 3)))
         return '{0}@{1}'.format(name, domain)
 
 
@@ -462,7 +483,7 @@ class IPAddressField(BaseField):
 
     def __init__(self, version='both', **kwargs):
         if version not in self.SUPPORT_VERSIONS:
-            raise ValueError('{} version is not supported'.format(version))
+            raise ValueError(_('{} version is not supported').format(version))
         self.version = version
 
         kwargs.setdefault('strict', False)
@@ -475,10 +496,10 @@ class IPAddressField(BaseField):
             raise exceptions.FieldValidationError(str(e))
         if self.version == 'ipv4' and value.version() != 4:
             raise exceptions.FieldValidationError(
-                'expected an ipv4 address, got {}'.format(value.strNormal()))
+                _('expected an ipv4 address, got {}').format(value.strNormal()))
         if self.version == 'ipv6' and value.version() != 6:
             raise exceptions.FieldValidationError(
-                'expected an ipv6 address, got {}'.format(value.strNormal()))
+                -('expected an ipv6 address, got {}').format(value.strNormal()))
         return value
 
     def to_presentation(self, value):
@@ -500,8 +521,8 @@ class IPAddressField(BaseField):
 class URLField(StringField):
     FIELD_TYPE_NAME = 'url'
     PARAMS = []
-    SCHEMAS = ('http', 'http')
-    
+    SCHEMAS = ('http', 'https')
+
     def __init__(self, **kwargs):
         kwargs['strict'] = True
         super(URLField, self).__init__(min_length=0, **kwargs)
@@ -510,9 +531,9 @@ class URLField(StringField):
         value = self._validate_type(value)
         url = urlparse.urlparse(value)
         if url.scheme not in self.SCHEMAS:
-            raise exceptions.FieldValidationError('schema is lost')
+            raise exceptions.FieldValidationError(_('schema is lost'))
         if url.hostname == '':
-            raise exceptions.FieldValidationError('hostname is lost')
+            raise exceptions.FieldValidationError(_('hostname is lost'))
         return url.geturl()
 
     def mock_data(self):
@@ -534,7 +555,7 @@ class EnumField(BaseField):
     def _validate(self, value):
         if value not in self.choices:
             raise exceptions.FieldValidationError(
-                '{!r} not in the choices'.format(value))
+                _('{!r} not in the choices').format(value))
         return value
 
     def mock_data(self):
@@ -566,6 +587,12 @@ class DictField(BaseField):
             value = copy.deepcopy(value)
         return value
 
+    def to_dict(self):
+        d = super(DictField, self).to_dict()
+        if d['validator'] is not None:
+            d['validator'] = d['validator'].to_dict()
+        return d
+
     def mock_data(self):
         if self.validator:
             return self.validator.mock_data()
@@ -580,7 +607,8 @@ class ListField(BaseField):
 
     def __init__(self, field=None, min_length=0, max_length=None, **kwargs):
         if field is not None and not isinstance(field, BaseField):
-            raise ValueError('field param expect a instance of BaseField, but got {!r}'.format(field))
+            raise ValueError(
+                _('field param expect a instance of BaseField, but got {!r}').format(field))
         self.field = field
 
         self._check_value_range(min_length, max_length)
@@ -593,11 +621,11 @@ class ListField(BaseField):
         value = self._validate_type(value)
         if self.min_length is not None and len(value) < self.min_length:
             raise exceptions.FieldValidationError(
-                'this list has too few elements, min length is {}'.format(self.min_length))
+                _('this list has too few elements, min length is {}').format(self.min_length))
 
         if self.max_length is not None and len(value) > self.max_length:
             raise exceptions.FieldValidationError(
-                'this list has too many elements, max length is {}'.format(self.max_length))
+                _('this list has too many elements, max length is {}').format(self.max_length))
 
         if self.field:
             new_value = []
@@ -609,6 +637,18 @@ class ListField(BaseField):
             value = copy.deepcopy(value)
         return value
 
+    def to_dict(self):
+        d = super(ListField, self).to_dict()
+        if d['field'] is not None:
+            d['field'] = d['field'].to_dict()
+        return d
+
+    @classmethod
+    def from_dict(cls, params):
+        if 'field' in params and isinstance(params['field'], dict):
+            params['field'] = create_field(params['field'])
+        return super(ListField, cls).from_dict(params)
+
     def mock_data(self):
         min_ = self.min_length
         if min_ is None:
@@ -616,11 +656,11 @@ class ListField(BaseField):
         max_ = self.max_length
         if max_ is None:
             max_ = 10
-        length = random.choice(xrange(min_, max_))
+        length = random.choice(range(min_, max_))
 
         data = [None] * length
         if self.field:
-            for i in xrange(length):
+            for i in range(length):
                 data[i] = self.field.mock_data()
         return data
 
@@ -638,7 +678,7 @@ class TimestampField(IntegerField):
             return super(TimestampField, self)._validate(value)
         except exceptions.FieldValidationError as e:
             raise exceptions.FieldValidationError(
-                'Got wrong timestamp: {}'.format(value))
+                _('Got wrong timestamp: {}').format(value))
 
 
 class DatetimeField(BaseField):
@@ -646,12 +686,18 @@ class DatetimeField(BaseField):
     FIELD_TYPE_NAME = 'datetime'
     PARAMS = ['dt_format', 'tzinfo']
     DEFAULT_FORMAT = '%Y/%m/%d %H:%M:%S'
-    
 
     def __init__(self, dt_format=None, tzinfo=None, **kwargs):
         if dt_format is None:
             dt_format = self.DEFAULT_FORMAT
         self.dt_format = dt_format
+        if isinstance(tzinfo, six.string_types):
+            try:
+                import pytz
+            except ImportError as e:
+                raise ValueError(
+                    _('Cant create DatetimeField instance with tzinfo {}, please install pytz and try again').format(params['tzinfo']))
+            tzinfo = pytz.timezone(tzinfo)
         self.tzinfo = tzinfo
         kwargs.setdefault('strict', False)
         super(DatetimeField, self).__init__(**kwargs)
@@ -665,12 +711,12 @@ class DatetimeField(BaseField):
             else:
                 dt = self.INTERNAL_TYPE.strptime(value, self.dt_format)
                 if self.tzinfo:
-                    dt = dt.astimezone(self.tzinfo)
+                    dt = dt.replace(tzinfo=self.tzinfo)
                 return dt
         elif isinstance(value, six.integer_types):
             return self.INTERNAL_TYPE.fromtimestamp(value, tz=self.tzinfo)
         else:
-            raise ValueError()
+            raise ValueError(_('Got wrong datetime value: {}').format(value))
 
     def _validate(self, value):
         value = self._validate_type(value)
@@ -679,6 +725,12 @@ class DatetimeField(BaseField):
     def to_presentation(self, value):
         return value.strftime(self.dt_format)
 
+    def to_dict(self):
+        d = super(DatetimeField, self).to_dict()
+        if d['tzinfo'] is not None:
+            d['tzinfo'] = force_text(d['tzinfo'])
+        return d
+
     def mock_data(self):
         return self.INTERNAL_TYPE.fromtimestamp(random.randint(0, 2 ** 32 - 1))
 
@@ -688,7 +740,7 @@ class DateField(BaseField):
     FIELD_TYPE_NAME = 'date'
     PARAMS = ['dt_format']
     DEFAULT_FORMAT = '%Y/%m/%d'
-    
+
     def __init__(self, dt_format=None, **kwargs):
         if dt_format is None:
             dt_format = self.DEFAULT_FORMAT
